



if not exist %LIBRARY_INC%\\llvm\\Pass.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\LLVMSupport.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
lld-link /?
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
